import requests
from monosodium_glutamate import api_types
from monosodium_glutamate import users
from monosodium_glutamate import utils
import bs4


class Thread:
    def __init__(self, data: dict, api):
        self.api = api
        self.data = data
        self.id = data["id"]
        self.title = data["title"]
        self.author_id = data["creator"]
        self.updater_id = data["updater"]
        self.response_count = data["response_count"]
        self.is_sticky = data["is_sticky"]
        self.is_deleted = data["is_deleted"]
        self.created_at = utils.readable_date(data["created_at"])
        self.updated_at = utils.readable_date(data["updated_at"])
        self.category_id = data["category_id"]
        self.is_new = data["is_new"]

    def get_author_details(self):
        return users.get_user_from_id(self.data["creator"])

    def get_updater_details(self):
        return users.get_user_from_id(self.data["updater"])

    def get_responses(self):
        page = requests.get(f"{self.api}/forum_topics/{self.id}", headers=utils.user_agent).text
        soup = bs4.BeautifulSoup(page, features="lxml")
        responses = soup.find_all("article", {"class": "forum-post"})
        data = {"responses": []}
        for x, resp in enumerate(responses):
            message = resp.find("div", {"class": "styled-dtext"})
            try:
                avatar = resp.find("div", {"class": "post-thumb"})["data-id"]
            except TypeError:
                avatar = None
            content = ""
            for p in message.find_all("p"):
                content += p.get_text() + " "
            data["responses"].append({
                "id": resp["data-forum-post-id"],
                "creator_id": resp["data-creator-id"],
                "creator_name": resp["data-creator"],
                "creator_avatar_id": avatar,
                "is_deleted": resp["data-is-deleted"],
                "post_time": resp.find("div", {"class": "post-time"}).find("time")["datetime"],
                "content": content
            })
        return ResponseList(data, self.api)


class ResponseList:
    def __init__(self, data, api):
        self.api = api
        self.data = data
        self.count = len(self.data["responses"])

    def get_response_from_index(self, index):
        return Response(self.data["responses"][index], self.api)


class Response:
    def __init__(self, data: dict, api):
        self.api = api
        self.id = data["id"]
        self.creator_id = data["creator_id"]
        self.creator_name = data["creator_name"]
        self.creator_avatar_id = data["creator_avatar_id"]
        self.is_deleted = data["is_deleted"]
        self.post_time = utils.readable_date(data["post_time"])
        self.content = data["content"]


class ThreadList:
    def __init__(self, data, api):
        self.api = api
        self.raw = data
        self.count = len(self.raw["threads"])

    def get_thread_from_index(self, index: int):
        return Thread(self.raw["threads"][index], self.api)


def get_topic_listing(page: int = 1, api=api_types.e621):
    """
    Retrieves the forum topic listing.
    :param page: The page to be searched.
    :param api: The API used.
    :return: A ThreadList object with the data.
    """

    html = requests.get(api + f"/forum_topics?page={page}", headers=utils.user_agent).text
    soup = bs4.BeautifulSoup(html, features="lxml")
    threads = soup.find_all("tr")
    threads.pop(0)
    # As there is no API for the listing, parse the HTML from the listing and get all the data
    # This is over 19 seconds faster than sending requests to the API for each thread
    data = {"threads": []}
    for thread in threads:
        data["threads"].append({
            "is_sticky": thread.find("span", {"class": "sticky"}) is not None,
            "is_new": thread.find("span", {"class": "new"}) is not None,
            "is_deleted": thread["data-topic-is-deleted"] == "true",
            "title": thread.find("a", {"class": "forum-post-link"}).string,
            "id": int(thread.find("a", {"class": "forum-post-link"})["href"].strip("/forum_topics/")),
            "creator": thread.find("td", {"class": "creator"}).find("a").string,
            "created_at": thread.find("td", {"class": "created-at"}).find("time")["datetime"],
            "updater": thread.find("td", {"class": "updater"}).find("a").string,
            "updated_at": thread.find("td", {"class": "updated-at"}).find("time")["datetime"],
            "response_count": thread.find_all("td")[len(thread.find_all("td")) - 1].string,
            "category_id": int(thread["class"][1].strip("forum-topic-category-"))
        })
    return ThreadList(data, api)


def get_thread(thread_id: int, api=api_types.e621):
    data = requests.get(f"{api}/forum_topics/{thread_id}.json", headers=utils.user_agent).json()
    return Thread(data, api)
