import subprocess
import warnings
import PIL
import requests
import urllib.request
import urllib.error
from PIL import Image
from shutil import which
from monosodium_glutamate import api_types
from monosodium_glutamate import utils
import os
import sys
import tempfile
if sys.platform == "win32":
    import win32api
    import win32print


opener = urllib.request.build_opener()
opener.addheaders = [('User-Agent', utils.user_agent["User-Agent"])]
urllib.request.install_opener(opener)


class Listing:
    """
    This is the object that will contain the list of posts from functions like get_posts().
    You only need to create an object using this if you are planning to interact with the API in different ways.
    :param data: the JSON data returned from the API
    """
    def __init__(self, data: dict, api):
        self.api = api
        self.data = data
        self.count = len(data["posts"])

    def get_post_from_index(self, index: int):
        """
        This function gets the post data from a certain index, and returns a Post object of the data.
        :param index: This is the numeric index of the post.
        :return: The Post object with the data of the post.
        """
        post = self.data["posts"][index]
        return Post(post, self.api)


def get_popular_posts(api=api_types.e621):
    """
    Gets the current popular posts on e621 or e926.
    **Please note that e926 will still contain NSFW content, as the API doesn't filter it.**
    :param api: The API you would like to use, default api_types.e621.
    :return: A Listing object of the popular posts.
    """
    popular = f"{api}/explore/posts/popular.json"
    data = requests.get(popular, headers=utils.user_agent).json()
    return Listing(data, api)


def get_posts(tags=None, api=api_types.e621):
    if tags is None:
        tags = []
    tag_list = ""
    for i in tags:
        tag_list += i + "+"
    posts = f"{api}/posts.json?tags={tag_list}"
    data = requests.get(posts, headers=utils.user_agent).json()
    return Listing(data, api)


def get_post_from_id(post_id: int, api=api_types.e621):
    post = f"{api}/posts/{post_id}.json"
    data = requests.get(post, headers=utils.user_agent).json()
    return Post(data["post"], api)


def create_post(
                login,
                api_key,
                tag_string,
                file,
                rating,
                source,
                api=api_types.e621,
                direct_url=None,
                description: str = None,
                parent_id=None,
                referer_url=None,
                md5_confirmation=None,
                as_pending=None
                ):
    """
    Post Creation

    -----

    **This was heavily based off of the create function in the e621 Python package.**
    **I'm unsure how to use this, as I do not have an account, however, I know you need to generate an API key.**

    I don't know what half of these parameters do, so I'm guessing on most of them.
    :param login: I think this is probably a sort of client ID
    :param api_key: the API key for the client
    :param tag_string: the string of tags for the post
    :param file: the file, maybe a url, I have no idea
    :param rating: The rating of the post, 's', 'e', or 'q'.
    :param source: the source of the post, optional
    :param api: api to use, default e621
    :param direct_url: file url? no idea
    :param description: the description of the post
    :param parent_id: the ID of the parent post, if any
    :param referer_url: not sure
    :param md5_confirmation: also not sure
    :param as_pending: probably just set this to true or something
    :return: the void of your soul
    """
    requests.post(api + "uploads.json", params={
                  "login": login,
                  "api_key": api_key,
                  "tag_string": tag_string,
                  "file": file,
                  "rating": rating, "source": source,
                  "direct_url": direct_url,
                  "description": description,
                  "parent_id": parent_id,
                  "referer_url": referer_url,
                  "md5_confirmation": md5_confirmation,
                  "as_pending": as_pending
                  })


class Post:
    def __init__(self, post: dict, api):
        self.raw = post
        # To make it easier for the end user, as well as to use this data
        self.type = post["file"]["ext"]
        # Failsafe post retrieval without an account
        self.file = utils.get_file_from_md5(post["file"]["md5"], self.type, api)
        self.id = post["id"]
        self.created_at = utils.readable_date(post["created_at"])
        self.updated_at = utils.readable_date(post["updated_at"])
        self.score = post["score"]["total"]
        self.tags = post["tags"]["general"]
        self.species = post["tags"]["species"]
        self.characters = post["tags"]["character"]
        self.copyright = post["tags"]["copyright"]
        self.artist = post["tags"]["artist"]
        self.rating = post["rating"]
        self.favorites = post["fav_count"]
        self.sources = post["sources"]
        self.pools = post["pools"]
        self.description = post["description"]
        self.duration = post["duration"]
        self.file_md5 = post["file"]["md5"]

    def save(self, filename: str):
        urllib.request.urlretrieve(self.file, f"{filename}.{self.type}")

    def show(self):
        try:
            im = Image.open(requests.get(self.file, headers=utils.user_agent, stream=True).raw)
            im.show()
        except PIL.UnidentifiedImageError:
            if self.type == "webm":
                if which("vlc") is None:
                    warnings.warn("VLC is not installed. Video posts cannot be opened. "
                                  "Download it from https://www.videolan.org/vlc/")
                else:
                    subprocess.call(["vlc", self.file, "--loop"])
            if self.type == "swf":
                utils.ruffle(self.file)

    def open(self):
        try:
            im = Image.open(requests.get(self.file, headers=utils.user_agent, stream=True).raw)
            return im
        except PIL.UnidentifiedImageError:
            warnings.warn("The post could not be opened, as it is not an image.")

    def print(self):
        if self.type == "png" or self.type == "jpg":
            temp_dir = tempfile.mkdtemp()
            urllib.request.urlretrieve(self.file, f"{temp_dir}/temp.{self.type}")
            if sys.platform == "win32":
                win32api.ShellExecute(
                    0,
                    "print",
                    f"{temp_dir}/temp.{self.type}",
                    f'/d:"{win32print.GetDefaultPrinter()}"',
                    ".",
                    0)
            else:
                subprocess.call(["lpr", f"{temp_dir}/temp.{self.type}"])
            os.remove(f"{temp_dir}/temp.{self.type}")
        else:
            warnings.warn("This file is not an image, therefore it cannot be printed.")

    def is_nsfw(self):
        if self.rating == "s":
            return False
        else:
            return True
